/*
 *  photoRecorder.cpp
 *  _crono3
 *
 *  Created by trollback on 4/12/11.
 *  Copyright 2011 __MyCompanyName__. All rights reserved.
 *
 */

#include "photoRecorder.h"

void PhotoRecorder::init(int x, int y, int w,int h) {
	
	Tweenzor::init();
	
	xpos = (float)x;
	ypos = (float)y;
	width = w;
	height = h;
	isRecording = false;
	currentframe = 0;
			
	bufferThreshold = 3;
	
	framestep    = 3;
	framerest    = 2;
	framecounter = 0;

    border_width = 10;

    max_frames = 300;
	
}

void PhotoRecorder::drawInfo(ofTrueTypeFont * font){
	ofSetColor(0xFFFFFF);
	string cnt = "r: " + ofToString(row, 0) + " c: " + ofToString(column, 0);
    font->drawString(cnt, xpos, ypos+20);  
}  

void PhotoRecorder::setSize(int w, int h) {
	width = w;
	height = h;
}

void PhotoRecorder::setPosition(int x, int y, bool animate) {
	
	if(x < 0) x = 0;
	if(y < 0) y = 0;

	if(animate){
				
		new_x = (float)x;
		new_y = (float)y;	
		
		Tweenzor::add( &xpos, xpos, new_x, ofRandom(0.f, .2f), ofRandom(0.f, .15f), EASE_IN_SINE);
		Tweenzor::getRecentTween()->addListener( Tween::COMPLETE, this, &PhotoRecorder::onCompleteX);

		Tweenzor::add( &ypos, ypos, new_y, ofRandom(0.f, .2f), ofRandom(0.f, .15f), EASE_OUT_SINE );
		Tweenzor::getRecentTween()->addListener( Tween::COMPLETE, this, &PhotoRecorder::onCompleteY);

	} else {
			
		xpos = (float)x;
		ypos = (float)y;
		
	}
		
	
}

void PhotoRecorder::setFromPixels( unsigned char* _pixels, int w, int h ) {
	
    unsigned char* px = new unsigned char [w*h*3];
    memcpy(px, _pixels, w*h*3);
	imageList.push_back(px);
    
    if(imageList.size() > max_frames){
        delete imageList[0];
        imageList.erase(imageList.begin());
    }
    
}


void PhotoRecorder::onCompleteX(float args) {
	xpos = new_x;
	cout << "complete x" << endl;
}

void PhotoRecorder::onCompleteY(float args) {
	ypos = new_y;
	cout << "complete y" << endl;
}


void PhotoRecorder::draw() {
   
	if(isRecording){

		if(imageList.size() > 0){
		
        } 	
        
        ofSetColor(0xFFFFFF);//116
        ofRect(xpos, ypos, width, border_width);
        ofRect(xpos, ypos+height-border_width, width, border_width);
        ofRect(xpos, ypos+border_width, border_width, height-border_width*2);
        ofRect(xpos+width-border_width, ypos+border_width, border_width, height-border_width*2);
	
    } else {
        
        ofSetColor(255,255,255,255);//116
		if(imageList.size() > bufferThreshold){
			//imageList[currentframe].draw(xpos, ypos, width, height);
            img.setFromPixels(imageList[currentframe], width, height, OF_IMAGE_GRAYSCALE);
            img.setImageType(OF_IMAGE_GRAYSCALE); 
            img.draw(xpos, ypos, width, height);
            
		} else if (imageList.size() > 0){
            img.setFromPixels(imageList[imageList.size()-1], width, height, OF_IMAGE_GRAYSCALE);
            img.setImageType(OF_IMAGE_GRAYSCALE); 
            img.draw(xpos, ypos, width, height);
		} else {
            ofSetColor(255,255,255, 200);//116
            ofRect(xpos, ypos, width, height);
            //ofSetColor(255,255,255, 100);
        }
	
    }
	
}

void PhotoRecorder::setCurrentFrame(int n){
    if(imageList.size() > 0){
    currentframe = n % imageList.size()-1;
    }
}

void PhotoRecorder::setOffset(int n){
    
    cout << "OFFSET N = " << n << endl;
    
    if(imageList.size() > 0){
        currentframe = (currentframe + n) % imageList.size();
    
    } else {
        currentframe = (currentframe + n);
        if(currentframe >= imageList.size())currentframe = 0;

    }
        
    //currentframe += n % imageList.size()-1;

    //cout << "offset " << currentframe << endl;
}

void PhotoRecorder::record(){
	
	isRecording = true;
   // currentframe = 0;
	//clear();
	
}

void PhotoRecorder::play(){	
    //currentframe = 0;
	isRecording = false;
}

int PhotoRecorder::getHeight(){
	return height;
}

int PhotoRecorder::getWidth(){
	return width;
}


int PhotoRecorder::getColumn(){
	return column;
}

int PhotoRecorder::getRow(){
	return row;
}

void PhotoRecorder::setColumn(int n){
	column = n;
}

void PhotoRecorder::setRow(int n){
	row = n;
}

void PhotoRecorder::shuffle(){
    
    int block = 3;
    vector <unsigned char*> tempList;// = new unsigned* char [width*height*3];; 
    
    vector <int> positionsList;
    
    int len = imageList.size();
    int mod = floor(len/block);
    
    for(int i = 0; i<mod; i++){
        positionsList.push_back(i);
    }
    random_shuffle(positionsList.begin(), positionsList.end());
    
    for(int j = 0; j<positionsList.size(); j++){
        int curr = positionsList[j];
        
        
        
        for(int k = (curr*block); k<(curr*block)+block; k++){
            
            cout << "k " << k << endl;
            
            if(k < imageList.size()){
                tempList.push_back(imageList[k]);
            }
        }
        
    }
    
    cout << "tempList " << tempList.size() << endl;
    
    imageList = tempList;
    
}

int PhotoRecorder::getNumFrames(){
	return imageList.size();
}
 

int PhotoRecorder::getXpos(){
	return xpos;
}

int PhotoRecorder::getYpos(){
	return ypos;
}

//--------------------------------------------------------------
void PhotoRecorder::update() {
    
	currentframe++;
    
    if(isRecording){
        cout << "current frame = " << currentframe << endl;
    }
    
    /*
    if((isRecording)&&(currentframe >= max_frames-1)){
        
        isRecording = false;
    } 
    */
	
    if(currentframe >= imageList.size())currentframe = 0;
	Tweenzor::update();
}

/*

ofImage PhotoRecorder::getImage(){
	return imageList[currentframe];
}

 */
//--------------------------------------------------------------
void PhotoRecorder::clear() {
	
	
	//inited = false;
	
	cout << "CLEAR" << getWidth() << " " << getHeight() << endl;
	
    for (int i = 0; i<imageList.size(); i++) {
        //free(imageList[i]);

        delete imageList[i];  
    }
    imageList.resize(0);
    
	//image.setFromPixels(pixels, getWidth(), getHeight(), OF_IMAGE_COLOR);
	
}